@@ -61,12 +61,14 @@ class progress(Thread):
         Thread.join(self, timeout)
 
 class install(Thread):
-    def __init__(self, source, target, persist, device=None):
+    def __init__(self, source, target, persist, device=None,
+                 allow_system_internal=False):
         Thread.__init__(self)
         self.source = source
         self.target = target
         self.persist = persist
         self.device = device
+        self.allow_system_internal = allow_system_internal
         self._stopevent = Event()
         self.progress_thread = None
         logging.debug('install thread source: %s' % source)
@@ -185,7 +187,7 @@ class install(Thread):
         if os.path.exists(ldlinux):
             os.remove(ldlinux)
 
-    def install_bootloader(self):
+    def install_bootloader(self, grub_location=''):
         logging.debug('install_bootloader')
         self.progress_pulse()
         self.progress_message(_('Installing the bootloader...'))
@@ -207,7 +209,8 @@ class install(Thread):
                 bus = dbus.SystemBus()
                 obj = bus.get_object('com.ubuntu.USBCreator',
                                      '/com/ubuntu/USBCreator')
-                obj.InstallBootloader(self.device,
+                obj.InstallBootloader(self.device, self.allow_system_internal,
+                                      grub_location,
                                       dbus_interface='com.ubuntu.USBCreator',
                                       timeout=MAX_DBUS_TIMEOUT)
             except dbus.DBusException:
@@ -236,27 +239,23 @@ class install(Thread):
         # Mangle the configuration files based on the options we've selected.
         import glob
         import lsb_release
+        try:
+            from debian import debian_support
+        except ImportError:
+            from debian_bundle import debian_support
         for filename in glob.iglob(os.path.join(self.target, 'syslinux', '*.cfg')):
             if os.path.basename(filename) == 'gfxboot.cfg':
                 continue
             f = None
             target_os_ver = None
-            our_os_ver = float(lsb_release.get_distro_information()['RELEASE'])
-            
+            our_os_ver = debian_support.Version(
+                lsb_release.get_distro_information()['RELEASE'])
+
             if os.path.exists(os.path.join(self.target, '.disk', 'info')):
                 with open(os.path.join(self.target, '.disk', 'info'),'r') as f:
                     contents = f.readline().split()
                 if len(contents) > 2:
-                    try:
-                        target_os_ver = float(contents[1])
-                    except ValueError:
-                        pass
-                    #might be a point release
-                    if not target_os_ver:
-                        try:
-                            target_os_ver = float(contents[1].rpartition('.')[0])
-                        except ValueError:
-                            pass
+                    target_os_ver = debian_support.Version(contents[1])
             try:
                 f = open(filename, 'r')
                 label = ''
@@ -277,20 +276,22 @@ class install(Thread):
                         #OS version specific mangles
                         #The syntax in syslinux changed with the version
                         #shipping in Ubuntu 10.10
-                        elif target_os_ver and our_os_ver and \
-                                                    target_os_ver != our_os_ver:
+                        elif (target_os_ver and our_os_ver and
+                              target_os_ver != our_os_ver):
+                            lucid = debian_support.Version('10.04')
+                            maverick = debian_support.Version('10.10')
                             #10.10 or newer image, burning on 10.04 or lower
-                            if command.lower() == 'ui' and \
-                                                    our_os_ver <= 10.04 and \
-                                                    target_os_ver >= 10.10:
+                            if (command.lower() == 'ui' and
+                                our_os_ver <= lucid and
+                                target_os_ver >= maverick):
                                 line.remove('ui')
                             #10.04 or earlier image, burning on 10.10 or higher
                             #Currently still broke.
-                            #elif command.lower() == 'gfxboot' and \
-                            #                        our_os_ver >= 10.10 and \
-                            #                        target_os_ver <= 10.04:
+                            #elif (command.lower() == 'gfxboot' and
+                            #      our_os_ver >= maverick and
+                            #      target_os_ver <= lucid):
                             #    line.insert(0, 'ui')
-                            
+
                     to_write.append(' '.join(line) + '\n')
                 f.close()
                 f = open(filename, 'w')
@@ -378,7 +379,7 @@ class install(Thread):
                 bus = dbus.SystemBus()
                 obj = bus.get_object('com.ubuntu.USBCreator',
                                      '/com/ubuntu/USBCreator')
-                obj.Image(self.source, self.device,
+                obj.Image(self.source, self.device, self.allow_system_internal,
                           dbus_interface='com.ubuntu.USBCreator',
                           timeout=MAX_DBUS_TIMEOUT)
             except dbus.DBusException:
@@ -419,7 +420,6 @@ class install(Thread):
         self.remove_extras()
         
         self.initialize_progress_thread()
-        self.install_bootloader()
 
         # Copy.
         
@@ -476,8 +476,14 @@ class install(Thread):
                 # and line[1] (HH:MM:SS).
                 logging.debug('mkdir %s' % os.path.join(self.target, line[2]))
                 os.mkdir(os.path.join(self.target, line[2]))
-        
-        self.mangle_syslinux()
+
+        grub = os.path.join(self.target, 'boot', 'grub', 'i386-pc')
+        if os.path.isdir(grub):
+            self.install_bootloader(grub)
+        else:
+            self.install_bootloader()
+            self.mangle_syslinux()
+
         self.create_persistence()
         self.sync()
 
@@ -507,7 +513,6 @@ class install(Thread):
         self.check()
         
         self.initialize_progress_thread()
-        self.install_bootloader()
 
         self.progress_message(_('Copying files...'))
         for dirpath, dirnames, filenames in os.walk(self.source):
@@ -542,8 +547,14 @@ class install(Thread):
                     if os.path.exists(targetpath):
                         os.unlink(targetpath)
                     self.copy_file(sourcepath, targetpath)
-        
-        self.mangle_syslinux()
+
+        grub = os.path.join(self.target, 'boot', 'grub', 'i386-pc')
+        if os.path.isdir(grub):
+            self.install_bootloader(grub)
+        else:
+            self.install_bootloader()
+            self.mangle_syslinux()
+
         self.create_persistence()
         self.sync()
     
